import Box from "@mui/material/Box";
import { EditableValue } from "./EditableValue";
import { moniness, moninessRadius } from "./state";
import Slider from "@mui/material/Slider";
import { refreshSimilarCalendarPriceChartData } from "./actions";

function MoninessChooser() {
  return (
    <Slider
      fullWidth
      label="Strike % From Underlying Price"
      value={moniness.value}
      valueLabelDisplay="on"
      min={0}
      max={10}
      step={1}
      onChange={(e, value) => {
        moniness.value = value as number;
      }}
      onChangeCommitted={(e, value) => {
        refreshSimilarCalendarPriceChartData();
      }}
      InputProps={{ endAdornment: "%" }}
    />
  );
}

function MoninessRadiusChooser() {
  return (
    <Slider
      fullWidth
      label="Strike % Radius"
      value={moninessRadius.value}
      valueLabelDisplay="on"
      min={0}
      max={10}
      step={1}
      onChange={(e, value) => {
        moninessRadius.value = value as number;
      }}
      onChangeCommitted={(e, value) => {
        refreshSimilarCalendarPriceChartData();
      }}
      InputProps={{ endAdornment: "%" }}
    />
  );
}

/** This is its own component so that sliding the slider with the mouse is
 * smoother. Preact detects reads from the "slider" signal values, and
 * associates them with the component that read them and redraws that component.
 * If this was not its own component, it would redraw the entire UI. It was very
 * slow. */
export function EditableStrike() {
  return (
    <EditableValue
      text={`${moniness.value.toFixed(1)}±${moninessRadius.value.toFixed(2)}`}
    >
      <Box sx={{ minWidth: "20em" }}>
        <MoninessChooser />
        <MoninessRadiusChooser />
      </Box>
    </EditableValue>
  );
}
